//+------------------------------------------------------------------+
//|                                                          UOS.mq5 |
//|                        Copyright 2016, MetaQuotes Software Corp. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2016, MetaQuotes Software Corp."
#property link      "https://www.mql5.com"
#property version   "1.00"
#property indicator_separate_window
//--- indicator settings
#property indicator_separate_window
#property indicator_buffers 6
#property indicator_plots   1
#property indicator_type1   DRAW_LINE
#property indicator_color1  clrDodgerBlue
//--- input parameters
input int FastPeriod=7;     // Fast ATR period
input int MediPeriod=14;    // Middle ATR period
input int SlowPeriod=28;    // Slow ATR period
input int FastK=4;          // Fast K
input int MediK=2;          // Middle K
input int SlowK=1;          // Slow K
//--- indicator buffers
double    TRBuffer[];
double    FastATRBuffer[];
double    MediATRBuffer[];
double    SlowATRBuffer[];
double    BPBuffer[];
double    UOBuffer[];
//--- indicator handles
int       ExtFastATRhandle;
int       ExtMiddleATRhandle;
int       ExtSlowATRhandle;
//--- global variable
double    divider;
int       maxPeriod;
#include <MovingAverages.mqh>
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
void OnInit()
  {
//--- indicator buffers mapping
   SetIndexBuffer(0,UOBuffer,INDICATOR_DATA);
   SetIndexBuffer(1,TRBuffer,INDICATOR_CALCULATIONS);
   SetIndexBuffer(2,FastATRBuffer,INDICATOR_CALCULATIONS);
   SetIndexBuffer(3,MediATRBuffer,INDICATOR_CALCULATIONS);
   SetIndexBuffer(4,SlowATRBuffer,INDICATOR_CALCULATIONS);
   SetIndexBuffer(5,BPBuffer,INDICATOR_CALCULATIONS);
//--- set accuracy
   IndicatorSetInteger(INDICATOR_DIGITS,2);
//--- set levels
   IndicatorSetInteger(INDICATOR_LEVELS,2);
   IndicatorSetDouble(INDICATOR_LEVELVALUE,0,30);
   IndicatorSetDouble(INDICATOR_LEVELVALUE,1,70);
//--- set maximum and minimum for subwindow 
   IndicatorSetDouble(INDICATOR_MINIMUM,0);
   IndicatorSetDouble(INDICATOR_MAXIMUM,100);
//--- set first bar from which index will be drawn
   PlotIndexSetInteger(0,PLOT_DRAW_BEGIN,SlowPeriod-1);
//--- name for DataWindow and indicator subwindow label
   IndicatorSetString(INDICATOR_SHORTNAME,StringFormat("UOS(%d,%d,%d)",FastPeriod,MediPeriod,SlowPeriod));
//---
   divider=FastK+MediK+SlowK;
   maxPeriod=SlowPeriod;
   if(maxPeriod<MediPeriod) 
      maxPeriod=MediPeriod;
   if(maxPeriod<FastPeriod)   
      maxPeriod=FastPeriod;
//--- initialization done
  }
//+------------------------------------------------------------------+
//| Ultimate Oscillator                                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
   if(prev_calculated>rates_total || prev_calculated<0)
      return(0);
   int toCount=rates_total-prev_calculated;
   if(prev_calculated>0)
      toCount++;
//--- check for rates count
   if(UOS(rates_total,prev_calculated,high,low,close)!=toCount)
      return(0);
//--- return value of prev_calculated for next call
   return(rates_total);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int UOS(const int rates_total,
        const int prev_calculated,
        const double &high[],
        const double &low[],
        const double &close[])
  {
//--- check for bars count
   if(rates_total<=SlowPeriod || FastPeriod<=1)
      return(0);
//--- declare variables
   int begin,count=0;
//--- initial zero
   if(prev_calculated<1)
     {
      TRBuffer[0]=FastATRBuffer[0]=MediATRBuffer[0]=SlowATRBuffer[0]=
      BPBuffer[0]=UOBuffer[0]=0.0;
      begin=SlowPeriod;
      double firstValue=0.0;
      for(int i=1;i<=SlowPeriod;i++)
        {
         TRBuffer[i]=MathMax(high[i],close[i-1])-MathMin(low[i],close[i-1]);
         firstValue+=TRBuffer[i];
         BPBuffer[i]=close[i]-MathMin(low[i],close[i-1]);
         UOBuffer[i]=0.0;
        }    
      firstValue/=SlowPeriod;
      SlowATRBuffer[SlowPeriod]=firstValue;
      firstValue=0.0;
      for(int i=SlowPeriod-FastPeriod;i<=SlowPeriod;i++)
         firstValue+=TRBuffer[i];
      firstValue/=FastPeriod;
      FastATRBuffer[SlowPeriod]=firstValue;
      firstValue=0.0;
      for(int i=SlowPeriod-MediPeriod;i<=SlowPeriod;i++)
         firstValue+=TRBuffer[i];
      firstValue/=MediPeriod;
      MediATRBuffer[SlowPeriod]=firstValue;
      begin=count=SlowPeriod+1;
     }
   else 
      begin=prev_calculated-1;
//--- calculate true low and ATR
   for(int i=begin;i<rates_total && !IsStopped();i++)
     {
      TRBuffer[i]=MathMax(high[i],close[i-1])-MathMin(low[i],close[i-1]);
      FastATRBuffer[i]=FastATRBuffer[i-1]+(TRBuffer[i]-TRBuffer[i-FastPeriod])/FastPeriod;
      MediATRBuffer[i]=MediATRBuffer[i-1]+(TRBuffer[i]-TRBuffer[i-MediPeriod])/MediPeriod;
      SlowATRBuffer[i]=SlowATRBuffer[i-1]+(TRBuffer[i]-TRBuffer[i-SlowPeriod])/SlowPeriod;
      BPBuffer[i]=close[i]-MathMin(low[i],close[i-1]);
     }
//--- the main loop of calculations
   double rawUOs;
   for(int i=begin;i<rates_total && !IsStopped();i++)
     {
      if(FastATRBuffer[i]!=0.0 && MediATRBuffer[i]!=0.0 && SlowATRBuffer[i]!=0.0)
        {
         rawUOs=FastK*SimpleMA(i,FastPeriod,BPBuffer)/FastATRBuffer[i]+
                MediK*SimpleMA(i,MediPeriod,BPBuffer)/MediATRBuffer[i]+
                SlowK*SimpleMA(i,SlowPeriod,BPBuffer)/SlowATRBuffer[i];
         UOBuffer[i]=rawUOs/divider*100;
        }
      else
         UOBuffer[i]=UOBuffer[i-1];
      count++;
     }
//--- wpr done. return count.
   return(count);
  }  
//+------------------------------------------------------------------+
